'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _kueScheduler = require('kue-scheduler');

var _kueScheduler2 = _interopRequireDefault(_kueScheduler);

var _basicAuthConnect = require('basic-auth-connect');

var _basicAuthConnect2 = _interopRequireDefault(_basicAuthConnect);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * AnyTVKue class injects additional helper functions on kue-scheduler
 * @example
 * const kue = require('anytv-kue')(config.app.KUE);
 * const Queue = kue.createQueue(config.app.KUE_OPTIONS);
 * kue.setup(Queue); // adds error listeners and requeue previous jobs
 */
var AnyTVKue = function () {

    /**
     * Creates an instance of AnyTVKue class
     * @param {object} [config] Config object that setups the Queue object
     */
    function AnyTVKue(config) {
        _classCallCheck(this, AnyTVKue);

        _lodash2.default.merge(_config2.default, config || {});
        _lodash2.default.defaults(this, _kueScheduler2.default);
    }

    /**
     * Creates a Queue object with specific options
     * @param {object} [options] Options to be passed to Queue object
     * @return {object} Queue instance
     */


    _createClass(AnyTVKue, [{
        key: 'createQueue',
        value: function createQueue(options) {
            var self = this;
            options = options || {};

            this.queue = _kueScheduler2.default.createQueue(options);

            if (this.queue.create.isCustom) {
                return this.queue;
            }

            this.queue._create = this.queue.create;

            this.queue.create = function () {
                var params = arguments[1] || {};

                if (!params.title) {
                    params.title = JSON.stringify(params);
                    arguments[1] = params;
                }
                var createResult = self.queue._create.apply(this, arguments);

                createResult._save = createResult.save;

                createResult.save = function () {
                    var remove_on_complete = 'remove_on_complete' in options ? options.remove_on_complete : _config2.default.remove_on_complete;
                    createResult.removeOnComplete(remove_on_complete);
                    createResult._save.apply(this, arguments);
                };

                return createResult;
            };

            this.queue.create.isCustom = true;

            /**
             * Patch created job's backoff function to support custom backoff types
             */
            var _createJob = this.queue.createJob;

            this.queue.createJob = function (name, payload) {

                var job = _createJob.call(this, name, payload);

                job.backoff = function (backoff_type) {

                    // default implementation is a getter when no arguments passed
                    // added for backwards compatibility
                    if (arguments.length === 0) {
                        return job._backoff;
                    }

                    var backoff = void 0;
                    switch (backoff_type) {

                        /**
                         * Custom callback functions
                         * Executed by eval() so cannot contain variables i.e. can't use config
                         * https://github.com/Automattic/kue#failure-backoff
                         */

                        // 2 mins, 4 mins, 8m mins...
                        case 'fixed_doubling':
                            backoff = function backoff(attempts) {
                                return Math.pow(2, attempts) * 1000 * 60;
                            };
                            break;

                        // custom exponential based on initial delay
                        // starts at 2 seconds when there's no initial delay
                        case 'delay_doubling':
                            backoff = '(attempts, delay) => delay ? delay * 2 : ' + _config2.default.default_delay + ';';
                            break;

                        default:
                            backoff = backoff_type;
                            break;
                    }

                    job._backoff = backoff;
                    return job;
                };

                return job;
            };

            return this.queue;
        }

        /**
         * Creates a function that can be used for activating KUE UI app
         * @param {object} app Express server
         * @returns {function} function
         * @example
         * activateUI(app, username, password)(route)
         * @example
         * activateUI(app, authMiddleWare)(route)
         * @example
         * activateUI(app)(route)
         */

    }, {
        key: 'activateUI',
        value: function activateUI(app) {
            var _arguments = arguments;


            if (!app || !_lodash2.default.isFunction(app.use)) {
                throw new Error('Invalid Argument');
            }

            switch (arguments.length) {
                case 1:
                    return function (route) {
                        app.use(route, _kueScheduler2.default.app);
                    };
                case 2:
                    return function (route) {
                        route = route || '/kue';

                        app.use(route, _arguments[1]);
                        app.use(route, _kueScheduler2.default.app);
                    };
                case 3:
                    return function (route) {
                        route = route || '/kue';

                        app.use(route, (0, _basicAuthConnect2.default)(_arguments[1], _arguments[2]));
                        app.use(route, _kueScheduler2.default.app);
                    };
            }
        }

        /**
         * Setups event listeners and re-queues previous jobs
         * @param {object} target Queue instance
         * @param {object} [callbacks] Object containing error, failed, sigterm handlers
         */

    }, {
        key: 'setup',
        value: function setup(target, callbacks) {
            callbacks = callbacks || {};

            target.on('error', callbacks.error || function (err) {
                _winston2.default.log('error', 'QUEUE ERROR:', err);
            });

            target.on('job failed', callbacks.failed || function (err) {
                _winston2.default.log('error', 'FAILED JOB:', err);
            });

            process.once('SIGTERM', callbacks.sigterm || function (sig) {
                _winston2.default.log('SIGTERM', sig);
                target.shutdown(_config2.default.shutdownTimer || 5000, function (err) {
                    _winston2.default.log('error', 'Kue shutdown:', err);
                    process.exit(0);
                });
            });

            var status = ['active', 'inactive'];

            //requeue all active and inactive jobs
            status.forEach(function (stat) {
                target[stat](callbacks[stat] || function (err, ids) {
                    ids.forEach(function (id) {
                        _kueScheduler2.default.Job.get(id, function (_err, job) {
                            if (job) {
                                job.inactive();
                            }
                        });
                    });
                });
            });
        }

        /**
         * Removes all jobs with specified job_name or status
         * @param {string} job_name Job name
         * @param {string} [status='complete'] Job status
         */

    }, {
        key: 'cleanup',
        value: function cleanup(job_name, status) {
            _kueScheduler2.default.Job.rangeByType(job_name, status || 'complete', 0, -1, 'asc', function (err, selectedJobs) {
                if (selectedJobs && selectedJobs.length) {
                    selectedJobs.forEach(function (job) {
                        job.remove();
                    });
                }
            });
        }

        /**
         * Returns config
         * @return {object} Config object
         */

    }, {
        key: 'getConfig',
        value: function getConfig() {
            return _config2.default;
        }
    }]);

    return AnyTVKue;
}();

exports.default = AnyTVKue;